<h3 class="donation">For Enterprise</h3>

<p>
Professionally-supported {{ project }} is available with the
<a href="https://tidelift.com/subscription/pkg/pypi-{{ project }}?utm_source=pypi-{{ project }}&utm_medium=referral">Tidelift Subscription</a>.
</p>

<h3>Download</h3>

<p>Current version: <b>{{ version }}</b></p>
<p>Get Setuptools from the <a href="https://pypi.org/project/setuptools/"> Python Package Index</a>

<h3>Questions? Suggestions? Contributions?</h3>

<p>Visit the <a href="{{ package_url }}">Project page</a> </p>
